@@ -1,17 +1,18 @@
 __version__ = "0.1"
 
+import datetime
+import json
 import os
 import unittest
-import numpy as np
-import json
-import datetime
-import pandas as pd
-from bson.objectid import ObjectId
 from enum import Enum
 
+import numpy as np
+import pandas as pd
+from bson.objectid import ObjectId
+
+from monty.json import MontyDecoder, MontyEncoder, MSONable, _load_redirect, jsanitize
+
 from . import __version__ as tests_version
-from monty.json import MSONable, MontyEncoder, MontyDecoder, jsanitize
-from monty.json import _load_redirect
 
 test_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "test_files")
 
@@ -96,6 +97,16 @@ class ClassContainingDataFrame(MSONable):
         self.df = df
 
 
+class ClassContainingSeries(MSONable):
+    def __init__(self, s):
+        self.s = s
+
+
+class ClassContainingNumpyArray(MSONable):
+    def __init__(self, np_a):
+        self.np_a = np_a
+
+
 class MSONableTest(unittest.TestCase):
     def setUp(self):
         self.good_cls = GoodMSONClass
@@ -141,7 +152,7 @@ class MSONableTest(unittest.TestCase):
         self.assertRaises(NotImplementedError, obj.as_dict)
         obj = self.auto_mson(2, 3)
         d = obj.as_dict()
-        objd = self.auto_mson.from_dict(d)
+        self.auto_mson.from_dict(d)
 
     def test_unsafe_hash(self):
         GMC = GoodMSONClass
@@ -149,11 +160,20 @@ class MSONableTest(unittest.TestCase):
         b_dict = {"first": GMC(3, 3.0, "three"), "second": GMC(4, 4.0, "four")}
         c_list_dict_list = [
             {
-                "list1": [GMC(5, 5.0, "five"), GMC(6, 6.0, "six"), GMC(7, 7.0, "seven"),],
+                "list1": [
+                    GMC(5, 5.0, "five"),
+                    GMC(6, 6.0, "six"),
+                    GMC(7, 7.0, "seven"),
+                ],
                 "list2": [GMC(8, 8.0, "eight")],
             },
             {
-                "list3": [GMC(9, 9.0, "nine"), GMC(10, 10.0, "ten"), GMC(11, 11.0, "eleven"), GMC(12, 12.0, "twelve"),],
+                "list3": [
+                    GMC(9, 9.0, "nine"),
+                    GMC(10, 10.0, "ten"),
+                    GMC(11, 11.0, "eleven"),
+                    GMC(12, 12.0, "twelve"),
+                ],
                 "list4": [GMC(13, 13.0, "thirteen"), GMC(14, 14.0, "fourteen")],
                 "list5": [GMC(15, 15.0, "fifteen")],
             },
@@ -161,7 +181,8 @@ class MSONableTest(unittest.TestCase):
         obj = GoodNestedMSONClass(a_list=a_list, b_dict=b_dict, c_list_dict_list=c_list_dict_list)
 
         self.assertEqual(
-            a_list[0].unsafe_hash().hexdigest(), "ea44de0e2ef627be582282c02c48e94de0d58ec6",
+            a_list[0].unsafe_hash().hexdigest(),
+            "ea44de0e2ef627be582282c02c48e94de0d58ec6",
         )
         self.assertEqual(obj.unsafe_hash().hexdigest(), "44204c8da394e878f7562c9aa2e37c2177f28b81")
 
@@ -176,11 +197,20 @@ class MSONableTest(unittest.TestCase):
         b_dict = {"first": GMC(3, 3.0, "three"), "second": GMC(4, 4.0, "four")}
         c_list_dict_list = [
             {
-                "list1": [GMC(5, 5.0, "five"), GMC(6, 6.0, "six"), GMC(7, 7.0, "seven"),],
+                "list1": [
+                    GMC(5, 5.0, "five"),
+                    GMC(6, 6.0, "six"),
+                    GMC(7, 7.0, "seven"),
+                ],
                 "list2": [GMC(8, 8.0, "eight")],
             },
             {
-                "list3": [GMC(9, 9.0, "nine"), GMC(10, 10.0, "ten"), GMC(11, 11.0, "eleven"), GMC(12, 12.0, "twelve"),],
+                "list3": [
+                    GMC(9, 9.0, "nine"),
+                    GMC(10, 10.0, "ten"),
+                    GMC(11, 11.0, "eleven"),
+                    GMC(12, 12.0, "twelve"),
+                ],
                 "list4": [GMC(13, 13.0, "thirteen"), GMC(14, 14.0, "fourteen")],
                 "list5": [GMC(15, 15.0, "fifteen")],
             },
@@ -264,7 +294,7 @@ class JsonTest(unittest.TestCase):
         self.assertEqual(type(d["dt"]), datetime.datetime)
 
     def test_uuid(self):
-        from uuid import uuid4, UUID
+        from uuid import UUID, uuid4
 
         uuid = uuid4()
         jsonstr = json.dumps(uuid, cls=MontyEncoder)
@@ -277,6 +307,12 @@ class JsonTest(unittest.TestCase):
         d = json.loads(jsonstr, cls=MontyDecoder)
         self.assertEqual(type(d["uuid"]), UUID)
 
+    def test_nan(self):
+        x = [float("NaN")]
+        djson = json.dumps(x, cls=MontyEncoder)
+        d = json.loads(djson)
+        self.assertEqual(type(d[0]), float)
+
     def test_numpy(self):
         x = np.array([1, 2, 3], dtype="int64")
         self.assertRaises(TypeError, json.dumps, x)
@@ -319,6 +355,25 @@ class JsonTest(unittest.TestCase):
         d = jsanitize(x, strict=True)
         assert type(d["energies"][0]) == float
 
+        # Test data nested in a class
+        x = np.array([[1 + 1j, 2 + 1j], [3 + 1j, 4 + 1j]], dtype="complex64")
+        cls = ClassContainingNumpyArray(np_a={"a": [{"b": x}]})
+
+        d = json.loads(json.dumps(cls, cls=MontyEncoder))
+
+        self.assertEqual(d["np_a"]["a"][0]["b"]["@module"], "numpy")
+        self.assertEqual(d["np_a"]["a"][0]["b"]["@class"], "array")
+        self.assertEqual(
+            d["np_a"]["a"][0]["b"]["data"],
+            [[[1.0, 2.0], [3.0, 4.0]], [[1.0, 1.0], [1.0, 1.0]]],
+        )
+        self.assertEqual(d["np_a"]["a"][0]["b"]["dtype"], "complex64")
+
+        obj = ClassContainingNumpyArray.from_dict(d)
+        self.assertIsInstance(obj, ClassContainingNumpyArray)
+        self.assertIsInstance(obj.np_a["a"][0]["b"], np.ndarray)
+        self.assertEqual(obj.np_a["a"][0]["b"][0][1], 2 + 1j)
+
     def test_pandas(self):
 
         cls = ClassContainingDataFrame(df=pd.DataFrame([{"a": 1, "b": 1}, {"a": 1, "b": 2}]))
@@ -333,6 +388,30 @@ class JsonTest(unittest.TestCase):
         self.assertIsInstance(obj.df, pd.DataFrame)
         self.assertEqual(list(obj.df.a), [1, 1])
 
+        cls = ClassContainingSeries(s=pd.Series({"a": [1, 2, 3], "b": [4, 5, 6]}))
+
+        d = json.loads(MontyEncoder().encode(cls))
+
+        self.assertEqual(d["s"]["@module"], "pandas")
+        self.assertEqual(d["s"]["@class"], "Series")
+
+        obj = ClassContainingSeries.from_dict(d)
+        self.assertIsInstance(obj, ClassContainingSeries)
+        self.assertIsInstance(obj.s, pd.Series)
+        self.assertEqual(list(obj.s.a), [1, 2, 3])
+
+        cls = ClassContainingSeries(s={"df": [pd.Series({"a": [1, 2, 3], "b": [4, 5, 6]})]})
+
+        d = json.loads(MontyEncoder().encode(cls))
+
+        self.assertEqual(d["s"]["df"][0]["@module"], "pandas")
+        self.assertEqual(d["s"]["df"][0]["@class"], "Series")
+
+        obj = ClassContainingSeries.from_dict(d)
+        self.assertIsInstance(obj, ClassContainingSeries)
+        self.assertIsInstance(obj.s["df"][0], pd.Series)
+        self.assertEqual(list(obj.s["df"][0].a), [1, 2, 3])
+
     def test_callable(self):
         instance = MethodSerializationClass(a=1)
         for function in [
@@ -408,6 +487,9 @@ class JsonTest(unittest.TestCase):
         clean_strict = jsanitize(d, strict=True)
         self.assertEqual(clean_strict["hello"]["a"], 1)
         self.assertEqual(clean_strict["hello"]["b"], 2)
+        clean_recursive_msonable = jsanitize(d, recursive_msonable=True)
+        self.assertEqual(clean_recursive_msonable["hello"]["a"], 1)
+        self.assertEqual(clean_recursive_msonable["hello"]["b"], 2)
 
         d = {"dt": datetime.datetime.now()}
         clean = jsanitize(d)
@@ -480,6 +562,15 @@ class JsonTest(unittest.TestCase):
         clean = jsanitize(d, strict=True)
         self.assertTrue("@class" in clean["c"])
 
+        # test on pandas
+        df = pd.DataFrame([{"a": 1, "b": 1}, {"a": 1, "b": 2}])
+        clean = jsanitize(df)
+        self.assertEqual(clean, df.to_dict())
+
+        s = pd.Series({"a": [1, 2, 3], "b": [4, 5, 6]})
+        clean = jsanitize(s)
+        self.assertEqual(clean, s.to_dict())
+
     def test_redirect(self):
         MSONable.REDIRECT["tests.test_json"] = {"test_class": {"@class": "GoodMSONClass", "@module": "tests.test_json"}}
 
@@ -501,7 +592,8 @@ class JsonTest(unittest.TestCase):
     def test_redirect_settings_file(self):
         data = _load_redirect(os.path.join(test_dir, "test_settings.yaml"))
         self.assertEqual(
-            data, {"old_module": {"old_class": {"@class": "new_class", "@module": "new_module"}}},
+            data,
+            {"old_module": {"old_class": {"@class": "new_class", "@module": "new_module"}}},
         )
 
     def test_pydantic_integrations(self):
